// -------------------------------------------------------
//
// Author: Thomas Lenart    
// Email:  thomas.lenart@es.lth.se
// Date:   March 2003
//
// -------------------------------------------------------

#include "main.h"

#ifdef DEBUG_ALIAS
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#endif
#include "alias.h"

#if NEW_CODE == 0
	static const fixed_point table_cs[8] =	{
		#if DATA_WIDTH == 32
			#include "table_cs"
		#else
			#include "table_cs_16"
		#endif
	};
	static const fixed_point table_ca[8] =	{
		#if DATA_WIDTH == 32
			#include "table_ca"
		#else
			#include "table_ca_16"
		#endif
	};
#else
	static const fixed_point table_cs[8] =	{
		#if DATA_WIDTH == 32
			#include "table_antialias_cs_32"
		#else
			#include "table_antialias_cs_16"
		#endif
	};
	static const fixed_point table_ca[8] =	{
		#if DATA_WIDTH == 32
			#include "table_antialias_ca_32"
		#else
			#include "table_antialias_ca_16"
		#endif
	};
	static const fixed_point table_neg_ca[8] =	{
		#if DATA_WIDTH == 32
			#include "table_antialias_neg_ca_32"
		#else
			#include "table_antialias_neg_ca_16"
		#endif
	};
#endif
void alias_reduction(fixed_point xr[MAX_SB*MAX_SS], fixed_point hybridIn[MAX_SB*MAX_SS], CT_Channel *channel_info)
{
	int i, sb, ss, sblim, k1, k17;
		
	#ifdef DEBUG_ALIAS
	FILE* file;
	if (ch_count == TARGET_CHANNEL || STEREO == 1) {
	#if DATA_WIDTH == 32
		file = fopen("debug_data/debug_alias_input_32.txt", OUTPUT_FILE_TYPE);
	#else
		file = fopen("debug_data/debug_alias_input_16.txt", OUTPUT_FILE_TYPE);
	#endif
	fprintf(file, "// This is frame %d, gr %d, ch %d, block_type = %d\n", frame_count, gr_count, ch_count, channel_info->block_type);
	fprintf(file, "-3: %d (%0*X)\n", channel_info->mixed_block_flag, DATA_WIDTH/4, channel_info->mixed_block_flag);
	fprintf(file, "-2: %d (%0*X)\n", channel_info->window_switching_flag, DATA_WIDTH/4, channel_info->window_switching_flag);
	fprintf(file, "-1: %d (%0*X)\n", channel_info->block_type, DATA_WIDTH/4, channel_info->block_type);
	for (i = 0; i < MAX_SB*MAX_SS; i++) {
		fprintf(file, "%2d: %8d (%04X)\n", i, xr[i], xr[i] & 0x0000FFFF);
	}
	fclose(file);

	#if DATA_WIDTH == 32
		file = fopen("debug_data/debug_alias_output_32.txt", OUTPUT_FILE_TYPE);
	#else
		file = fopen("debug_data/debug_alias_output_16.txt", OUTPUT_FILE_TYPE);
	#endif
	fprintf(file, "// This is frame %d, gr %d, ch %d, block_type = %d\n", frame_count, gr_count, ch_count, channel_info->block_type);
	}
	#endif

		#if NEW_CODE == 1
 		for(i=0;i<MAX_SB*MAX_SS;i++)				// !!! Don't do this if not nessesary. Move it!
			hybridIn[i] = xr[i];    
		#endif
           
    if (channel_info->window_switching_flag && (channel_info->block_type == 2) && !channel_info->mixed_block_flag ) {
//    if ((channel_info->block_type == 2) && !channel_info->mixed_block_flag ) {

		#if NEW_CODE == 0
 		for(i=0;i<MAX_SB*MAX_SS;i++)				// !!! Don't do this if not nessesary. Move it!
			hybridIn[i] = xr[i];    
		#endif
		#ifdef DEBUG_ALIAS
		if (ch_count == TARGET_CHANNEL || STEREO == 1) {
		fprintf(file, "//Bypass\n");
		for (i = 0; i < 576; ++i) 
			fprintf(file, "%2d: %8d (%04X)\n", i, hybridIn[i], hybridIn[i] & 0x0000FFFF);
		fclose(file);
		}
		#endif
		return;
	}

    if (channel_info->window_switching_flag && channel_info->mixed_block_flag && (channel_info->block_type == 2))
		sblim = 1;
	else
		sblim = MAX_SB-1;

	#if NEW_CODE == 0
	for(ss = 0; ss < 10; ss++) {				// copy samples not assigned below
		hybridIn[ss] = xr[ss];
		hybridIn[MAX_SB*MAX_SS-ss-1] = xr[MAX_SB*MAX_SS-ss-1];
	}
	#endif
	
	k17 = 17;
	k1 = MAX_SS;
	for(sb = 0; sb < sblim; sb++) {  
		for(ss = 0; ss < 8; ss++) {
			#if NEW_CODE == 1
				hybridIn[k17-ss] = (fix_mul(xr[k17-ss], table_cs[ss])) + (fix_mul(xr[k1+ss], table_neg_ca[ss]));
			#else
				hybridIn[k17-ss] = (fix_mul(xr[k17-ss], table_cs[ss])) - (fix_mul(xr[k1+ss], table_ca[ss]));
			#endif
			hybridIn[k1+ss]  = (fix_mul(xr[k1+ss], table_cs[ss])) + (fix_mul(xr[k17-ss], table_ca[ss]));
		}
		hybridIn[k1+8] = xr[k1+8];
		hybridIn[k1+9] = xr[k1+9];
		k17 += MAX_SS;
		k1 += MAX_SS;
	}
	
	#ifdef DEBUG_ALIAS
	if (ch_count == TARGET_CHANNEL || STEREO == 1) {
	fprintf(file, "// Block type = %d\n", channel_info->block_type);
	for (i = 0; i < 576; ++i) 
		fprintf(file, "%2d: %8d (%04X)\n", i, hybridIn[i], hybridIn[i] & 0x0000FFFF);
	fclose(file);
	}
	#endif
}
