// -------------------------------------------------------
//
// Author: Thomas Lenart    
// Email:  thomas.lenart@es.lth.se
// Date:   March 2003
//
// -------------------------------------------------------

#include <stdio.h>
#include "main.h"
#include "decoder.h"
#include "mpeg.h"
#include "wave.h"

int (*ct_decoder_input_register)(CT_InputModule *input_module);		// global input module function declaration
int (*ct_decoder_output_register)(CT_OutputModule *output_module);	// global output module function declaration

void decoder_init(int (*input_register)(CT_InputModule *input_module), int (*output_register)(CT_OutputModule *output_module))
{
	ct_decoder_input_register = input_register;					// save input function reference before calling decoder_run
	ct_decoder_output_register = output_register;	
}

void decoder_run(char* filename)
{
	CT_InputModule  input_module;
	CT_OutputModule output_module;
	CT_Decoder		*mpeg;
	CT_MPEGInfo*	mpeg_frame = 0;
	int temp_flag;

	ct_decoder_input_register(&input_module);					// register the input device
	ct_decoder_output_register(&output_module);					// register the output device

	mpeg = mpeg_stream_init(8192);								// initiate the mpeg stream

	if (output_module.open)	{									// if input_output function is assigned
		output_module.open(filename);									// call output_open
	}
	if (input_module.open)	{									// if input_open function is assigned
		input_module.open(filename);									// call input_open
	}
		
	// initiation finished
	// repeat until end of stream
	// 1. read from input device
	// 2. decode one frame
	// 3. write to output device (if output device not ready => use spin-lock)

	input_module.read(mpeg);
	input_module.read(mpeg);
	
	printf("Start decoder...\n");
	
	frame_count = 0;
	while (1) {													// while not end of stream
		#ifdef DEBUG
		printf("reading...\n");
		#endif

		temp_flag = input_module.read(mpeg);								// try to read data from the input module
	
		if (!buffer_empty(mpeg->input_buffer)) {					// if input buffer is not empty
			if (!buffer_empty(mpeg->output_buffer) && frame_count != 0)	{				// if output buffer is not empty
				output_module.write(mpeg);							// write data to the output buffer
			}	
			frame_count++;
			printf("frame extracted: # %d\n", frame_count);
			mpeg_frame = mpeg_stream_decode(mpeg);			// decode one mpeg frame
		
			#ifdef DEBUG
			printf("frame processed\n");
			#if STOP_PER_FRAME == 1
				getchar();
			#endif
			#endif
		} else {
			if (temp_flag == CT_OK)
				break;
		}
	}

	if (!buffer_empty(mpeg->output_buffer))	{				// if output buffer is not empty
		last_frame = 1;
		output_module.write(mpeg);							// write data to the output buffer
	}	

	printf("Decoder done...\n");
	printf("Total # of frames decoded: %u\n", frame_count);
	printf("------------------------------------------------\n\n");
	if (input_module.close)										// if input_close function is assigned
		input_module.close();									// call input_close
	if (output_module.close)									// if input_close function is assigned
		output_module.close();									// call input_close
}
