// -------------------------------------------------------
//
// Author: Thomas Lenart    
// Email:  thomas.lenart@es.lth.se
// Date:   March 2003
//
// -------------------------------------------------------

#ifdef _WIN32
#include <stdio.h>
#endif
#include "main.h"
#include "fixpoint.h"

//#ifdef _WIN32
fixed_point quick_hi, quick_lo;

void print_fix(fixed_point x) {
	char fix_string[64];
	int  i;
	
	for (i = 0; i < DATA_WIDTH; i++)
		fix_string[DATA_WIDTH-1-i] = DATA_WIDTH+16 + ((x >> i) & 1);

	fix_string[i] = '\0';
//	printf("%s - %d\n", fix_string, x);
}

fixed_point double_to_fixed(double x) {
	unsigned int *addr =  (unsigned int*)(&x);
	unsigned int y_lo;
	unsigned int y_hi;
	char sign;
	int shift;
	unsigned int exponent;
	unsigned int significand;

	y_lo = (unsigned int)(*addr);
	addr++;
	y_hi = (unsigned int)(*addr);
	sign = (char)(y_hi >> 31);
	exponent = (unsigned int)(y_hi >> 20) & (0x7FF);
	significand = (unsigned int)((y_lo >> 20) + (y_hi << 12));
	significand >>= 2;
	significand |= 0x40000000;

//	significand >>= INTEGER_BITS - 1;

/*	printf("size: %d \n", sizeof(y_lo));
	printf("y: \n");
	print_fix(y_hi);
	print_fix(y_lo);
	print_fix(significand);
	printf("\n");

	printf("double %f \n", x);
	printf("ulong %u \n", y_hi);
	printf("ulong %u \n", y_lo);
	printf("sign %d, exp %d, significand %d \n", sign, exponent, significand); */

	shift = 1023 - exponent;
	if (shift > 0) {
		shift = shift + INTEGER_BITS - 1;
		if (shift < 32)
			significand >>= shift;
		else
			significand = 0;
	}
	else {
		shift = -shift - INTEGER_BITS + 1;
		if (shift > 0)
			significand <<= shift;
		else
			significand >>= -shift;
	}

	if (sign)
		significand = (significand ^ ((2 << 32) - 1)) + 1;

//	print_fix(significand);

	return significand;
}

double fixed_to_double(fixed_point x) {
  double y;
  unsigned int *addr = (unsigned int*)(&y);
  char sign = 0;
  unsigned int exponent;
  unsigned int significand;
  unsigned int y_hi, y_lo;
  char pos = 0;
  
  if (DATA_WIDTH == 32) {
	  if (x >> 31) {
  	  x = (x ^ ((2 << 32) - 1)) + 1;
    	sign = 1;
	  }
  
  	//	print_fix(x);
	  while ((!(x & 0x80000000)) && (pos < 32)){
  	  x <<= 1;
    	pos+=1;
	  }
  	//	printf("pos %d\n", pos);
  
	  significand = (x - 0x80000000) << 1;
  	exponent = 1023 - pos + INTEGER_BITS;
  
	  y_hi = (sign << 31) + (exponent << 20) + (significand >> 12);
  	y_lo = (significand << 20);
  } else {
  	if (x >> 16) {
  	  x = (x ^ ((2 << 16) - 1)) + 1;
    	sign = 1;
	  }
  
  	//	print_fix(x);
	  while ((!(x & 0x8000)) && (pos < 16)){
  	  x <<= 1;
    	pos+=1;
	  }
  	//	printf("pos %d\n", pos);
  
	  significand = (x - 0x8000) << 1;
  	exponent = 16 - pos + INTEGER_BITS;
  
	  y_hi = (sign << 16) + (exponent << 10) + (significand >> 6);
  	y_lo = (significand << 10);
  }
  
  //	print_fix(y_hi);
  //	print_fix(y_lo);
  
  *addr = y_lo;
  addr++;
  *addr = y_hi;
  
  return y;
}

signed short fixed_to_pcm_sample(fixed_point x) {
/*	if (x & (0x80000000)) {								// if < 0 
		//if ((x & (1 << (15 - INTEGER_BITS))) == 0)				// if LSB-1 = 1
			x = fix_add(x, 1 << (15 - INTEGER_BITS));	// round
	}
	else {
		if (x & (1 << (15 - INTEGER_BITS)))
			x = fix_add(x, 1 << (15 - INTEGER_BITS));
	}*/
	x = fix_add(x, 1 << (15 - INTEGER_BITS));	// round
	return (x >> (16 - INTEGER_BITS));
}

//#endif

#if NEW_MULT == 1
#if DATA_WIDTH == 32

fixed_point fix_mul(int x, int y) {
	long long temp;
	fixed_point result;
	
//	temp = ((long long)x) * y;

//	result = temp >> (32 - INTEGER_BITS - 1);
	
	result = ((fixed_point)((((long long)(x))*(y)) >> (32 - INTEGER_BITS - 1) ));

	if (((x > 0 && y < 0) || (x < 0 && y > 0)) && result > 0) result = 0x80000000;
  if (((x < 0 && y < 0) || (x > 0 && y > 0)) && result < 0) result = 0x7FFFFFFF;
  
// 	if(temp & 0x000000001FFFC000 > 0) {
// 		if (result >= 0) result += 1;
// 		else result -= 1;
// 	}
		
	return result;
}

#else

fixed_point fix_mul(fixed_point x, fixed_point y) {
	int temp;
	fixed_point result;
	
	temp = ((int)x) * y;
	
	result = temp >> (16 - INTEGER_BITS - 1);

	if (((x > 0 && y < 0) || (x < 0 && y > 0)) && result > 0) result = 0x8000;
  if (((x < 0 && y < 0) || (x > 0 && y > 0)) && result < 0) result = 0x7FFF;

// 	if(temp & 0x00001FC0 > 0) {
// 		if (result >= 0) result += 1;
// 		else result -= 1;
// 	}
		
	return result;
}

#endif
#endif
