// -------------------------------------------------------
//
// Author: Thomas Lenart    
// Email:  thomas.lenart@es.lth.se
// Date:   March 2003
//
// -------------------------------------------------------
#include "main.h"

#ifdef DEBUG_IMDCT
#include <stdio.h>
#include <limits.h>
#endif

#include "imdct.h"
#include "constants.h"

//                       [ch][sub-band][sb-sample]
fixed_point  prev_block[2][MAX_SB][MAX_SS]; //[2][32][18] 1152 words = 576 words/ch.


#ifdef DEBUG_IMDCT
static int num_executions = -1;

static int frame  = 0;
#endif

// 144 words
#if NEW_CODE == 0
static const fixed_point win_fix[4][36] = { 
	#if DATA_WIDTH == 32
		#include "table_windows"
	#else
		#include "table_windows_16"
	#endif
};
#else
static const fixed_point imdct_window_0[36] = { 
	#if DATA_WIDTH == 32
		#include "table_imdct_window_0_32"
	#else
		#include "table_imdct_window_0_16"
	#endif
};
static const fixed_point imdct_window_1[36] = { 
	#if DATA_WIDTH == 32
		#include "table_imdct_window_1_32"
	#else
		#include "table_imdct_window_1_16"
	#endif
};
static const fixed_point imdct_window_2[12] = { 
	#if DATA_WIDTH == 32
		#include "table_imdct_window_2_32"
	#else
		#include "table_imdct_window_2_16"
	#endif
};
static const fixed_point imdct_window_3[36] = { 
	#if DATA_WIDTH == 32
		#include "table_imdct_window_3_32"
	#else
		#include "table_imdct_window_3_16"
	#endif
};
#endif

#if NEW_CODE == 1
static const fixed_point table_imdct_cos[36][18] =	{
	#if DATA_WIDTH == 32
		#include "table_imdct_cos_32"
	#else
		#include "table_imdct_cos_16"
	#endif
};

static const fixed_point table_imdct_short_cos[12][6] =	{
	#if DATA_WIDTH == 32
		#include "table_imdct_short_cos_32"
	#else
		#include "table_imdct_short_cos_16"
	#endif
};
#endif

void imdct_init(){
    int i,j,k;
    
    for(i = 0; i < 2; i++)
	for(j = 0; j < MAX_SB; j++)
	    for(k = 0; k < MAX_SS; k++)
		prev_block[i][j][k] = 0;
}


void imdct(fixed_point input[MAX_SB*MAX_SS], fixed_point output[MAX_SB*MAX_SS],
	   int ch, CT_Channel *channel_info)
{
    int		        ss, sb, block_type;
    fixed_point const*	window;
    fixed_point		rawout[36];
    fixed_point*	prev_line;
    fixed_point*	dct_in;
    fixed_point*	dct_out;
    int j, k;

#ifdef DEBUG_IMDCT
    FILE *file;
    FILE *icg_b4_win_fp, *ipg_b4_win_fp;
    #if DATA_WIDTH == 32
	    file = fopen("debug_data/debug_imdct_32.txt", OUTPUT_FILE_TYPE);
    #else
      file = fopen("debug_data/debug_imdct_16.txt", OUTPUT_FILE_TYPE);
		#endif
    #if DATA_WIDTH == 32
	   	icg_b4_win_fp = fopen("debug_data/debug_icg_b4_window_32.txt", OUTPUT_FILE_TYPE);
	   	ipg_b4_win_fp = fopen("debug_data/debug_ipg_b4_window_32.txt", OUTPUT_FILE_TYPE);
    #else
	   	icg_b4_win_fp = fopen("debug_data/debug_icg_b4_window_16.txt", OUTPUT_FILE_TYPE);
	   	ipg_b4_win_fp = fopen("debug_data/debug_ipg_b4_window_16.txt", OUTPUT_FILE_TYPE);
		#endif

    num_executions++;
    if (!num_executions)
    {
	fprintf(file, "Size of fixed_point is %ld bytes (%ld bits)\n", 
		(unsigned long)sizeof(fixed_point), 8*((unsigned long)sizeof(fixed_point)));
		#if DATA_WIDTH == 32
	fprintf(file, "MAX_FIXED_POINT = %d\n", INT_MAX);
	fprintf(file, "MIN_FIXED_POINT = %d\n", INT_MIN);    
	#else
	fprintf(file, "MAX_FIXED_POINT = %d\n", SHRT_MAX);
	fprintf(file, "MIN_FIXED_POINT = %d\n", SHRT_MIN);    
	#endif
    }
/*     fprintf(file, " --- Frame = %4d---\n", frame);     */
if (ch == TARGET_CHANNEL || STEREO == 1)
	fprintf(file, "// This is frame %d, gr %d, ch %d, block_type = %d\n", frame_count, gr_count, ch_count, channel_info->block_type);
#endif
// for block type 2

    for (sb = 0; sb < MAX_SB; sb++) { // Iterate over all 32 sub-bands.
	block_type = (channel_info->window_switching_flag && channel_info->mixed_block_flag && (sb < 2)) ? 0 : channel_info->block_type; 
    
	dct_in = &input[sb*MAX_SS]; // Pointer to the samples whithin the sub-band.
	dct_out = &output[sb];     
    
	prev_line = prev_block[ch][sb]; // Pointer to the previuos samples in the sub-band.
    
	if (block_type == 2) {  // includes windowing (the matrix requires it)
		/* Reads from dct_in[0-17] 
		 * Writes to rawout[0-35]
		 * Writes to dct_out[(0-17)*MAX_SB]
		 * prev_line[n] = rawout[n+18] 
		 */
 
					
		#if NEW_CODE == 1
			window = imdct_window_2;
			#if DATA_WIDTH == 32
				#include "imdct_short_gen_32_new.c" 
			#else
				#include "imdct_short_gen_16_new.c"
			#endif
		#else
			#if DATA_WIDTH == 32
				#include "imdct_short_gen.c" 
			#else
				#include "imdct_short_gen_16.c"
			#endif
		#endif
	} else {  // block_type != 2
		#if NEW_CODE == 0
	    window = win_fix[block_type];		// uses win_fix for windowing
		#else
			switch (block_type) {
			case 1: window = imdct_window_1; break;
			case 3: window = imdct_window_3; break;
			default: window = imdct_window_0; break;
			}
		#endif
		#if NEW_CODE == 1
			#ifdef DEBUG_IMDCT
				fprintf(icg_b4_win_fp, "// This is frame %d, gr %d, ch %d, sb %d\n", frame_count, gr_count, ch_count, sb);
				fprintf(ipg_b4_win_fp, "// This is frame %d, gr %d, ch %d, sb %d\n", frame_count, gr_count, ch_count, sb);
			#endif

			#if DATA_WIDTH == 32
				#include "imdct_gen_32_new.c" 
			#else
				#include "imdct_gen_16_new.c"
			#endif
		#else
			#if DATA_WIDTH == 32
				#include "imdct_gen.c" 
			#else
				#include "imdct_gen_16.c"
			#endif
		#endif
	}

#ifdef DEBUG_IMDCT
if (ch == TARGET_CHANNEL || STEREO == 1) {
	fprintf(file, "Sub-band %2d Block_type = window_type = %d\n", sb, block_type);
	if (block_type == 2) 
	    fprintf(file, "Short IMDCT\n");
 	fprintf(file, "dct_in\n");
	for(ss = 0; ss < MAX_SS; ss++) 
	    //fprintf(file, "[%2d] %34.31f\n", ss, new_fixed_to_double(dct_in[ss]));	
	    fprintf(file, "[%2d] %34d (%0*X)\n", ss, dct_in[ss], DATA_WIDTH/4, dct_in[ss] & MASK);	
	fprintf(file, "rawout\n"); 
	for(ss = 0; ss < 2*MAX_SS; ss++)
	{ 
	    //fprintf(file, "[%2d] %34.31f\n", ss, new_fixed_to_double(rawout[ss]));
	    fprintf(file, "[%2d] %34d (%0*X)\n", ss, rawout[ss], DATA_WIDTH/4, rawout[ss] & MASK);
	}
	fprintf(file, "dct_out\n"); 
	for(ss = 0; ss < MAX_SS; ss++)
	{ 
	    //fprintf(file, "[%2d] %34.31f\n", ss, new_fixed_to_double(rawout[ss]));
	    fprintf(file, "[%2d] %34d (%0*X)\n", ss, dct_out[ss*MAX_SB], DATA_WIDTH/4, dct_out[ss*MAX_SB] & MASK);
	}
	fprintf(file, "prev_line\n"); 
	for(ss = 0; ss < MAX_SS; ss++)
	{ 
	    //fprintf(file, "[%2d] %34.31f\n", ss, new_fixed_to_double(rawout[ss]));
	    fprintf(file, "[%2d] %34d (%0*X)\n", ss, prev_line[ss], DATA_WIDTH/4, prev_line[ss] & MASK);
	}
	/*	
	if (block_type != 2)
	{
	    fprintf(file, "Window\n");
	    for(ss = 0; ss < 36; ss++)
	    { 
		fprintf(file, "[%2d] %34d (%0*X)\n", ss, window[ss], DATA_WIDTH/4, window[ss] & MASK);
	    }
	    
	    fprintf(file, "After windowing\n");
	    for(ss = 0; ss < 36; ss++)
	    { 
		fprintf(file, "[%2d] %34d (%0*X)\n", ss, fix_mul(rawout[ss], window[ss]), DATA_WIDTH/4, fix_mul(rawout[ss], window[ss]) & MASK);
	    }
	}
*/
}
#endif
	
    } // End of the 32 sub-band iterations.
    
	#ifdef DEBUG_IMDCT    
		fclose(icg_b4_win_fp);
		fclose(ipg_b4_win_fp);
    fclose(file);

		if (ch_count == TARGET_CHANNEL || STEREO == 1) {
			#if DATA_WIDTH == 32
				file = fopen("debug_data/debug_icg_32.txt", OUTPUT_FILE_TYPE);
			#else
				file = fopen("debug_data/debug_icg_16.txt", OUTPUT_FILE_TYPE);
			#endif
			fprintf(file, "// This is frame %d, gr %d, ch %d\n", frame_count, gr_count, ch_count);
			for (ss = 0; ss < MAX_SS; ss++) {
				for (sb = 0; sb < MAX_SB; sb++) {
					fprintf(file, "%d: %d (%0*X)\n", ss*MAX_SB+sb, output[ss*MAX_SB+sb], DATA_WIDTH/4, output[ss*MAX_SB+sb] & MASK);
				}
			}
			fclose(file);
		}
		
		if (ch_count == TARGET_CHANNEL || STEREO == 1) {
			#if DATA_WIDTH == 32
				file = fopen("debug_data/debug_ipg_32.txt", OUTPUT_FILE_TYPE);
			#else
				file = fopen("debug_data/debug_ipg_16.txt", OUTPUT_FILE_TYPE);
			#endif
			fprintf(file, "// This is frame %d, gr %d, ch %d\n", frame_count, gr_count, ch_count);
			for (ss = 0; ss < MAX_SS; ss++) {
				for (sb = 0; sb < MAX_SB; sb++) {
					fprintf(file, "%d: %d (%0*X)\n", ss*MAX_SB+sb, prev_block[ch][sb][ss], DATA_WIDTH/4, prev_block[ch][sb][ss] & MASK);
				}
			}
			fclose(file);
		}

	#endif


    for (ss = 1; ss < MAX_SS; ss += 2) // {1, 3, 5, 7, ..., 15, 17} every odd side-band sample.
	for (sb = 1; sb < MAX_SB; sb += 2) // {1, 3, 5, 7, ..., 31} Odd sub-bands.
	    output[ss*MAX_SB+sb] = -output[ss*MAX_SB+sb];  // All odd subband samples in all odd subbands 

#ifdef DEBUG_IMDCT
    //fprintf(file, "--- New Frame ---\n");

    //for(sb = 0; sb < MAX_SB; sb++) 
    //{
//	fprintf(file, "Sub-band %2d\n", sb);
//	for(ss = 0; ss < MAX_SS; ss++) 
//	    fprintf(file, " input[%2d] = %12d = %33.30f\n", ss, input[ss*sb], 
//		    new_fixed_to_double(input[ss*sb]));
	//for(ss = 0; ss < MAX_SS; ss++) 
	//    fprintf(file, "output[%2d] = %12d = %33.30f\n", ss, output[ss*sb], 
	//	    new_fixed_to_double(output[ss*sb]));
//	for(ss = 0; ss < 36; ss++) 
//	    fprintf(file, "rawout[%2d] = %12d = %33.30f\n", ss, rawout[ss], 
//		    new_fixed_to_double(rawout[ss]));
//    }

    //for(ss=0; ss<MAX_SS*MAX_SB; ss++)
	//fprintf(file, "%d\n", output[ss]);
      //fprintf(file, "imdct[%d][%d] %.10f\n", sb, ss, fixed_to_double(dct_out[ss*32]));
//	fprintf(file, "sb = %2d output[%3d] = %12d = %33.30f\n", ss/MAX_SB, ss, output[ss], 
//		new_fixed_to_double(output[ss])); 
    frame++;
#endif
}
