// -------------------------------------------------------
//
// Author: Thomas Lenart    
// Email:  thomas.lenart@es.lth.se
// Date:   March 2003
//
// -------------------------------------------------------

#include <string.h>
#include <stdio.h>

#include "main.h"
#include "decoder.h"				// mp3 decoder
#include "modules/fileinput.h"			// file input module
#include "modules/fileoutput.h"			// file output module

int main(int argc, char* argv[])
{
	char filename[2000];
	
  if (argc == 2) {
  #ifdef GEN_MEM
			// for printing input data for requantizer
	#if GEN_TYPE == 4	
		argv[1] = TARGET_FILE;
	#endif
	#endif
	decoder_init(&file_input_register, &file_output_register);		

	start_frame_num = -1;

		sprintf(filename, "%s.mp3", argv[1]);
		printf("Decoding MP3 with %d bits: %s\n\n", DATA_WIDTH, filename);
		#if NEW_CODE == 1
			printf("Using new code\n");
		#else
			printf("Using original code\n");
		#endif
		
		#if NEW_MULT == 1
			printf("Using new Mult\n");
		#else
			printf("Using original Mult\n");
		#endif

mem_count = START_MEM;
#ifdef GEN_MEM
#if GEN_TYPE == 0
	f_mem_controller("alias_input");
	f_mem_open("alias_input");
#endif
#if GEN_TYPE == 1
	f_mem_controller("imdct_input");
	f_mem_open("imdct_input");
#endif
#if GEN_TYPE == 2
	f_mem_controller("subsyn_input");
	f_mem_open("subsyn_input");
#endif
#if GEN_TYPE == 3
	f_mem_controller("ac97_input");
	f_mem_open("ac97_input");
#endif
#if GEN_TYPE == 4
	f_mem_controller("requantizer_input");
	f_mem_open("requantizer_input");
#endif
#endif
#ifdef GEN_ETHERNET
	char eth_filename[2000];
	sprintf(eth_filename, "output_samples/ethernet/%s_left.txt", argv[1]);
	
	if ((eth_left_fp = fopen(eth_filename, "w")) == NULL) {
		printf("Error opening %s\n", eth_filename);
		return;
	}
	sprintf(eth_filename, "output_samples/ethernet/%s_right.txt", argv[1]);
	if ((eth_right_fp = fopen(eth_filename, "w")) == NULL) {
		printf("Error opening \n", eth_filename);
		return;
	}
#endif

	if ((temp_fp = fopen("temp.txt", "w")) == NULL) {
		printf("Error opening temp.txt\n");
		return;
	}
	#if NEWDATA == 1
	char data_filename[200];
	
	sprintf(data_filename, "../../MP3_noFloat/%s_left.txt", argv[1]);
	
	printf("Getting data from \"%s\" and ", data_filename);
	if ((left_data = fopen(data_filename, "r")) == NULL) {
		printf("Error opening %s\n", data_filename);
		return;
	}
	sprintf(data_filename, "../../MP3_noFloat/%s_right.txt", argv[1]);
	printf("\"%s\"...\n", data_filename);
	if ((right_data = fopen(data_filename, "r")) == NULL) {
		printf("Error opening %s\n", data_filename);
		return;
	}
	#endif

	srand( (unsigned)time( NULL ) );

	mixed_block_type_count = 0;
	is_count = 0;
	ms_count = 0;
	average_ms_limit = 0;
	average_rnsample = 0;
	average_lnsample = 0;
	average_cbmax = 0;
	last_frame = 0;


  	decoder_run(filename);
#ifdef GEN_MEM
  				f_mem_close();
#endif

	fclose(temp_fp);

#ifdef GEN_ETHERNET
	fclose(eth_left_fp);
	fclose(eth_right_fp);
#endif
	printf("\n\nmixed_block_type_count = %d\n", mixed_block_type_count);
// 	if (mixed_block_type_count > 0) getchar();
	printf("\nms_count = %d\n", ms_count);
// 	if (ms_count > 0) getchar();
	printf("\nis_count = %d\n", is_count);
// 	if (is_count > 0) getchar();
	printf("\naverage ms limit=%f\n", (float)average_ms_limit/(float)is_count);
	printf("\naverage rnsample=%f\n", (float)average_rnsample/(float)is_count);
	printf("\naverage lnsample=%f\n", (float)average_lnsample/(float)is_count);
	printf("\naverage cbmax=%f\n", (float)average_cbmax/(float)is_count);

  } else
  	printf("Usage: decoder <file.mp3>\n\n");
}
