#include "main.h"
#include <stdio.h>
#include <stdarg.h>
#include <math.h>

#include "print_mem.h"
#include "string.h"

int syn_init_count, param_init_count;
int syn_location_count, param_location_count;
char syn_mem_value[64];
char param_mem_value[64][80];
FILE *mop;
char mem_instance_name[100];
char *ori_instance_name;

void f_mem_controller_header();
void f_mem_controller_logic();
void f_mem_controller_instantiate(char *);
	
void f_mem_controller(char *instance_name) {
	char filename[200];

	sprintf(filename, "%s/memory_controller.v", MEM_PATH);
	printf("File opened: %s\n", filename);
	mop = fopen(filename, "w");
	
	ori_instance_name = instance_name;
	
	f_mem_controller_header();
	f_mem_controller_logic();
	f_mem_controller_instantiate(instance_name);
	
	fprintf(mop, "endmodule\n");
	fclose(mop);
}

void f_mem_controller_header() {
	int i;
	
	fprintf(mop, "// Author: Henry Ko\n");
	fprintf(mop, "\n");
	fprintf(mop, "`timescale 1 ns / 100 ps\n\n");
	fprintf(mop, "`include \"defines.v\"\n\n");
	fprintf(mop, "module memory_controller (\n");
	fprintf(mop, "\tCLOCK_I,\n");
	fprintf(mop, "\tRESETN_I,\n");
	fprintf(mop, "\tSTART_I,\n");
	#if STEREO == 0
		fprintf(mop, "\tADDRESS_O,\n");
		fprintf(mop, "\tDATA_O,\n");
		fprintf(mop, "\tWE_O,\n");
		fprintf(mop, "\tEN_O,\n");
		fprintf(mop, "\tBLOCK_TYPE_O,\n");
	#else
		fprintf(mop, "\tLEFT_ADDRESS_O,\n");
		fprintf(mop, "\tLEFT_DATA_O,\n");
		fprintf(mop, "\tLEFT_WE_O,\n");
		fprintf(mop, "\tLEFT_EN_O,\n");
		fprintf(mop, "\tLEFT_BLOCK_TYPE_O,\n");
		fprintf(mop, "\tRIGHT_ADDRESS_O,\n");
		fprintf(mop, "\tRIGHT_DATA_O,\n");
		fprintf(mop, "\tRIGHT_WE_O,\n");
		fprintf(mop, "\tRIGHT_EN_O,\n");
		fprintf(mop, "\tRIGHT_BLOCK_TYPE_O,\n");
	#endif
	fprintf(mop, "\tDONE_O\n");
	fprintf(mop, ");\n\n");
	
	fprintf(mop, "input CLOCK_I;\n");
	fprintf(mop, "input RESETN_I;\n");
	fprintf(mop, "input START_I;\n");
	#if STEREO == 0
	fprintf(mop, "output [`ADDRESS_WIDTH-1:0] ADDRESS_O;\n");
	fprintf(mop, "output [`DATA_WIDTH-1:0] DATA_O;\n");
	fprintf(mop, "reg [`DATA_WIDTH-1:0] DATA_O;\n");	
	fprintf(mop, "output WE_O;\n");
	fprintf(mop, "reg WE_O;\n");
	fprintf(mop, "output EN_O;\n");
	fprintf(mop, "output [1:0] BLOCK_TYPE_O;\n");
	fprintf(mop, "reg [1:0] BLOCK_TYPE_O;\n");

	#else
	fprintf(mop, "output [`ADDRESS_WIDTH-1:0] LEFT_ADDRESS_O;\n");
	fprintf(mop, "output [`DATA_WIDTH-1:0] LEFT_DATA_O;\n");
	fprintf(mop, "reg [`DATA_WIDTH-1:0] LEFT_DATA_O;\n");	
	fprintf(mop, "output LEFT_WE_O;\n");
	fprintf(mop, "reg LEFT_WE_O;\n");
	fprintf(mop, "output LEFT_EN_O;\n");
	fprintf(mop, "output [1:0] LEFT_BLOCK_TYPE_O;\n");
	fprintf(mop, "reg [1:0] LEFT_BLOCK_TYPE_O;\n");
	fprintf(mop, "output [`ADDRESS_WIDTH-1:0] RIGHT_ADDRESS_O;\n");
	fprintf(mop, "output [`DATA_WIDTH-1:0] RIGHT_DATA_O;\n");
	fprintf(mop, "reg [`DATA_WIDTH-1:0] RIGHT_DATA_O;\n");	
	fprintf(mop, "output RIGHT_WE_O;\n");
	fprintf(mop, "reg RIGHT_WE_O;\n");
	fprintf(mop, "output RIGHT_EN_O;\n");
	fprintf(mop, "output [1:0] RIGHT_BLOCK_TYPE_O;\n");
	fprintf(mop, "reg [1:0] RIGHT_BLOCK_TYPE_O;\n");
	#endif
	fprintf(mop, "output DONE_O;\n");	
	fprintf(mop, "reg DONE_O;\n\n");

	fprintf(mop, "reg [9:0] sample_counter;\n");	
	fprintf(mop, "reg [`ADDRESS_WIDTH-1:0] logical_address;\n");
	fprintf(mop, "reg [`ADDRESS_WIDTH-1:0] physical_address;\n");
	fprintf(mop, "reg [%d:0] memory_counter;\n\n", MEM_COUNTER_WIDTH-1);
	
	for (i = 0; i < NUM_MEM; i++) {
		fprintf(mop, "wire [%d:0] data_out_%02d;\n", DATA_WIDTH-1, i);
	}
	fprintf(mop,"\n");
	
	fprintf(mop, "parameter NUM_MEM = 'd%d;\n", NUM_MEM);
	fprintf(mop, "parameter NUM_SAMPLE = 'd%d;\n\n", NUM_SAMPLE);
	
	fprintf(mop, "reg [1:0] state;\n\n");
}

void f_mem_controller_logic() {
	int i;
	
	#if STEREO == 0
		fprintf(mop, "assign ADDRESS_O = logical_address;\n\n");
		fprintf(mop, "assign EN_O = ~DONE_O;\n\n");
	#else
		fprintf(mop, "assign LEFT_ADDRESS_O = logical_address;\n\n");
		fprintf(mop, "assign LEFT_EN_O = ~DONE_O;\n\n");
		fprintf(mop, "assign RIGHT_ADDRESS_O = logical_address;\n\n");
		fprintf(mop, "assign RIGHT_EN_O = ~DONE_O;\n\n");
	#endif
	
	fprintf(mop, "always @ (posedge CLOCK_I or negedge RESETN_I) begin\n");
	fprintf(mop, "\tif (RESETN_I == 1'b0) begin\n");
	fprintf(mop, "\t\tstate <= 'd0;\n");
	fprintf(mop, "\t\tDONE_O <= 1'b1;\n");
	#if STEREO == 0
	fprintf(mop, "\t\tWE_O <= 1'b0;\n");
	#else
	fprintf(mop, "\t\t\tLEFT_WE_O <= 1'b0;\n");
	fprintf(mop, "\t\t\tRIGHT_WE_O <= 1'b0;\n");
	#endif
	fprintf(mop, "\t\tlogical_address <= `ADDRESS_WIDTH'h0;\n");
	fprintf(mop, "\t\tphysical_address <= `ADDRESS_WIDTH'h0;\n");
	fprintf(mop, "\t\tmemory_counter <= %d'h0;\n", MEM_COUNTER_WIDTH);
	fprintf(mop, "\t\tsample_counter <= 10'h0;\n");
	fprintf(mop, "\tend else begin\n");
	fprintf(mop, "\t\tcase (state)\n");
	fprintf(mop, "\t\t'd0: begin\n");
	fprintf(mop, "\t\t\tDONE_O <= 1'b1;\n");
	#if STEREO == 0
	fprintf(mop, "\t\t\tWE_O <= 1'b0;\n\n");
	#else
	fprintf(mop, "\t\t\tLEFT_WE_O <= 1'b0;\n");
	fprintf(mop, "\t\t\tRIGHT_WE_O <= 1'b0;\n");
	#endif
	fprintf(mop, "\t\t\tif (logical_address >= NUM_SAMPLE-1) logical_address <= `ADDRESS_WIDTH'h0;\n\n");
	fprintf(mop, "\t\t\tif (START_I == 1'b1) state <= 'd1;\n");
	fprintf(mop, "\t\tend\n");
	fprintf(mop, "\t\t'd1: begin\n");
	fprintf(mop, "\t\t\tDONE_O <= 1'b0;\n");
	#if STEREO == 0
	fprintf(mop, "\t\t\tWE_O <= 1'b1;\n\n");
	#else
	fprintf(mop, "\t\t\tLEFT_WE_O <= 1'b1;\n");
	fprintf(mop, "\t\t\tRIGHT_WE_O <= 1'b1;\n");
	#endif
	fprintf(mop, "\t\t\tphysical_address <= physical_address + 1;\n");
	#if STEREO == 0
		fprintf(mop, "\t\t\tif (physical_address == `ADDRESS_WIDTH'h3ff) memory_counter <= memory_counter + 1;\n");
		fprintf(mop, "\t\t\tif (physical_address >= `ADDRESS_WIDTH'd%d && memory_counter == NUM_MEM-1) begin\n", 1024-(1024*NUM_MEM) % NUM_SAMPLE-1);
	#else
		fprintf(mop, "\t\t\tif (physical_address == `ADDRESS_WIDTH'h3ff) memory_counter <= memory_counter + 2;\n");
		fprintf(mop, "\t\t\tif (physical_address >= `ADDRESS_WIDTH'd%d && memory_counter == NUM_MEM-2) begin\n", 1024-(1024*(NUM_MEM/2)) % NUM_SAMPLE-1);
	#endif
	fprintf(mop, "\t\t\t\tphysical_address <= `ADDRESS_WIDTH'h0;\n");
	fprintf(mop, "\t\t\t\tmemory_counter <= %d'h0;\n", MEM_COUNTER_WIDTH);
	fprintf(mop, "\t\t\tend\n");
	fprintf(mop, "\n");	
	fprintf(mop, "\t\t\tstate <= 'd2;\n");
	fprintf(mop, "\t\tend\n");
	fprintf(mop, "\t\tdefault: begin\n");
	fprintf(mop, "\t\t\tDONE_O <= 1'b0;\n\n");
	fprintf(mop, "\t\t\tif (sample_counter < NUM_SAMPLE-1) begin\n");
	#if STEREO == 0
	fprintf(mop, "\t\t\t\tWE_O <= 1'b1;\n");
	#else
	fprintf(mop, "\t\t\tLEFT_WE_O <= 1'b1;\n");
	fprintf(mop, "\t\t\tRIGHT_WE_O <= 1'b1;\n");
	#endif
	fprintf(mop, "\t\t\t\tphysical_address <= physical_address + 1;\n");
	#if STEREO == 0
		fprintf(mop, "\t\t\t\tif (physical_address == `ADDRESS_WIDTH'h3ff) memory_counter <= memory_counter + 1;\n\n");
		fprintf(mop, "\t\t\tif (physical_address >= `ADDRESS_WIDTH'd%d && memory_counter == NUM_MEM-1) begin\n", 1024-(1024*NUM_MEM) % NUM_SAMPLE-1);
	#else
	fprintf(mop, "\t\t\t\tif (physical_address == `ADDRESS_WIDTH'h3ff) memory_counter <= memory_counter + 2;\n\n");
	fprintf(mop, "\t\t\tif (physical_address >= `ADDRESS_WIDTH'd%d && memory_counter == NUM_MEM-2) begin\n", 1024-(1024*(NUM_MEM/2)) % NUM_SAMPLE-1);
	#endif
	fprintf(mop, "\t\t\t\t\tphysical_address <= `ADDRESS_WIDTH'h0;\n");
	fprintf(mop, "\t\t\t\t\tmemory_counter <= %d'h0;\n", MEM_COUNTER_WIDTH);
	fprintf(mop, "\t\t\t\tend\n");
	fprintf(mop, "\n");	
	fprintf(mop, "\t\t\t\tlogical_address <= logical_address + 1;\n");
	fprintf(mop, "\t\t\t\tsample_counter <= sample_counter + 1;\n");
	fprintf(mop, "\n");
	fprintf(mop, "\t\t\t\tif (physical_address == 'd30) begin\n");
	fprintf(mop, "\t\t\t\t\tLEFT_BLOCK_TYPE_O <= LEFT_DATA_O[1:0];\n");
	fprintf(mop, "\t\t\t\t\tRIGHT_BLOCK_TYPE_O <= RIGHT_DATA_O[1:0];\n");
	fprintf(mop, "\t\t\t\tend\n");
	fprintf(mop, "\t\t\tend else begin\n");
	#if STEREO == 0
	fprintf(mop, "\t\t\t\tWE_O <= 1'b0;\n");
	#else
	fprintf(mop, "\t\t\tLEFT_WE_O <= 1'b0;\n");
	fprintf(mop, "\t\t\tRIGHT_WE_O <= 1'b0;\n");
	#endif
	fprintf(mop, "\t\t\t\tlogical_address <= logical_address + 1;\n");
	fprintf(mop, "\t\t\t\tsample_counter <= 0;\n\n");
	fprintf(mop, "\t\t\t\t state <= 'd0;\n");
	fprintf(mop, "\t\t\tend\n");
	fprintf(mop, "\t\tend\n");
	fprintf(mop, "\t\tendcase\n");
	fprintf(mop, "\tend\n");
	fprintf(mop, "end\n\n");
		
	fprintf(mop, "// select output from memory\n");
	fprintf(mop, "always @ (memory_counter");
	for (i = 0; i < NUM_MEM; i++) {
		fprintf(mop, " or data_out_%02d", i);
		if (i % 10 == 0) {
			fprintf(mop, "\n");
		}
	}
	fprintf(mop, ") begin\n");
	fprintf(mop, "\tcase (memory_counter)\n");
	#if STEREO == 0
		for (i = 0; i < NUM_MEM-1; i++) {
			fprintf(mop, "\t%d'd%d: DATA_O <= data_out_%02d;\n", MEM_COUNTER_WIDTH, i, i);
		}
		fprintf(mop, "\tdefault: DATA_O <= data_out_%02d;\n", NUM_MEM-1);
	#else
		for (i = 0; i < NUM_MEM-3; i+=2) {
			fprintf(mop, "\t%d'd%d: begin\n", MEM_COUNTER_WIDTH, i);
			fprintf(mop, "\t\tLEFT_DATA_O <= data_out_%02d;\n", i);
			fprintf(mop, "\t\tRIGHT_DATA_O <= data_out_%02d;\n", i+1);
			fprintf(mop, "\tend\n");
		}
		fprintf(mop, "\tdefault: begin\n");
		fprintf(mop, "\t\tLEFT_DATA_O <= data_out_%02d;\n", NUM_MEM-2);
		fprintf(mop, "\t\tRIGHT_DATA_O <= data_out_%02d;\n", NUM_MEM-1);
		fprintf(mop, "\tend\n");
	#endif
	fprintf(mop, "\tendcase\n");
	fprintf(mop, "end\n\n");
}

void f_mem_controller_instantiate(char *instance_name) {
	int i;
	
	for (i = 0; i < NUM_MEM; i++) {
		fprintf(mop, "%s_%02d data_memory_%02d (\n", instance_name, i, i);
		fprintf(mop, "\tCLOCK_I,\n");
		fprintf(mop, "\t1'b1,\n");
		fprintf(mop, "\tphysical_address,\n");
		fprintf(mop, "\tdata_out_%02d\n", i);
		fprintf(mop, ");\n\n");
	}
}

void f_mem_open(char *instance_name) {
	char filename[200];
	
	if (mem_count < NUM_MEM) {
		ori_instance_name = instance_name;
		sprintf(mem_instance_name, "%s_%02d", instance_name, mem_count);
		sprintf(filename, "%s/%s.v", MEM_PATH, mem_instance_name);
		printf("memory file opened: %s\n", filename);
		mop = fopen(filename, "w");
		f_mem_reset();
		f_mem_print_header(mem_instance_name);
	} else {
		f_mem_print_syn_remain();
		f_mem_print_param_remain();
		f_mem_close();
	}
}

void f_mem_reset() {
	int i;
	
	syn_init_count = 0;
	syn_location_count = 0;
	param_init_count = 0;
	param_location_count = 0;
	sprintf(syn_mem_value, "0000000000000000000000000000000000000000000000000000000000000000");
	
	for (i = 0; i < 64; i++) {
		sprintf(param_mem_value[i], "0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000");
	}
}

void f_mem_print_syn(fixed_point number) {
	char temp[4];
	int i;
		
	syn_location_count++;
	param_location_count++;
	
	if (number >= 0) {
		sprintf(temp, "%0*X", DATA_WIDTH/4, number);
	} else {
		sprintf(temp, "%0*X", DATA_WIDTH/4, number & MASK);
	}
	
	if (syn_location_count == 16) {
		for (i = 0; i < 4; i++) {
			syn_mem_value[i] = temp[i];
			param_mem_value[param_init_count][i] = temp[i];
		}
		
		f_mem_print("// synthesis attribute INIT_%02X of instance_%s_RAMB16_S18 is \"256'h%s\"\n", syn_init_count, mem_instance_name, syn_mem_value);
		sprintf(syn_mem_value, "0000000000000000000000000000000000000000000000000000000000000000");
		syn_init_count++;
		param_init_count++;
		
		if (syn_init_count == 0x40) {
			f_mem_print_param();
			f_mem_close();
			f_mem_open(ori_instance_name);
		} else {
			syn_location_count = 0;
			param_location_count = 0;
		}
	} else {
/* 	printf("haha: %02X, %d\n", syn_init_count, syn_location_count); */
		for (i = 0; i < 4; i++) {
			syn_mem_value[64-4*syn_location_count+i] = temp[i];
		}
		for (i = 0; i < 4; i++) {
			param_mem_value[param_init_count][80-5*param_location_count+i] = temp[i];
		}
		param_mem_value[param_init_count][80-5*param_location_count-1] = '_';
	}
}

void f_mem_print_syn_remain() {
	if (syn_location_count != 16) {
		f_mem_print("// synthesis attribute INIT_%02X of instance_%s_RAMB16_S18 is \"256'h%s\"\n", syn_init_count, mem_instance_name, syn_mem_value);
		sprintf(syn_mem_value, "0000000000000000000000000000000000000000000000000000000000000000");
		syn_init_count++;
		syn_location_count = 0;
	}
}

void f_mem_print_param() {
	int i;
	
	f_mem_print_syn_off();

	for (i = 0; i < param_init_count; i++) {
		f_mem_print("defparam instance_%s_RAMB16_S18.INIT_%02X = 256'h%s;\n", mem_instance_name, i, param_mem_value[i]);
	}
	
	f_mem_print_syn_on();
}

void f_mem_print_param_remain() {
	if (param_location_count != 16) {
		f_mem_print("defparam instance_%s_RAMB16_S18.INIT_%02X = 256'h%s;\n", mem_instance_name, param_init_count, param_mem_value[param_init_count]);
		sprintf(param_mem_value[param_init_count], "0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000_0000");
		param_init_count++;
		param_location_count = 0;
	}
}

void f_mem_print_header() {
	f_mem_print("// This file is generated by Henry Ko\n");
	f_mem_print("\n");
	f_mem_print("`include \"defines.v\"\n");
	f_mem_print("\n");
	f_mem_print("`timescale 1 ns/ 100 ps\n");
	f_mem_print("\n");
	f_mem_print("module %s (\n", mem_instance_name);
	f_mem_print("\tclock,\n");
	f_mem_print("\tmem_en,\n");
	f_mem_print("\taddress,\n");
	#if GEN_TYPE == 4
		f_mem_print("\twr_en,\n");
	#endif
	f_mem_print("\tdata_out\n");
	f_mem_print(");\n");
	f_mem_print("\n");
	f_mem_print("input clock;\n");
	f_mem_print("input mem_en;\n");
	f_mem_print("input [`ADDRESS_WIDTH-1:0] address;\n");
	#if GEN_TYPE == 4
		f_mem_print("input wr_en;\n");
	#endif
	f_mem_print("output [`DATA_WIDTH-1:0] data_out;\n");
	f_mem_print("\n");
	f_mem_print("// Instantiate the RAM\n");
	f_mem_print("RAMB16_S18 instance_%s_RAMB16_S18 (\n", mem_instance_name);
	f_mem_print("\t.DO(data_out),\n");
	f_mem_print("\t.DOP(),\n");
	f_mem_print("\t.ADDR(address),\n");
	f_mem_print("\t.CLK(clock),\n");
	f_mem_print("\t.DI(16'h0),\n");
	f_mem_print("\t.DIP(2'b00),\n");
	f_mem_print("\t.EN(mem_en),\n");
	f_mem_print("\t.SSR(1'b0),\n");
	#if GEN_TYPE == 4
		f_mem_print("\t.WE(wr_en)\n");
	#else
		f_mem_print("\t.WE(1'b0)\n");
	#endif
	f_mem_print(");\n");
	f_mem_print("\n");
}

void f_mem_print_end() {
	f_mem_print("endmodule\n");
}

void f_mem_print_syn_on() {
	f_mem_print("\n");
	f_mem_print("// synthesis translate_on\n");
	f_mem_print("\n");
}

void f_mem_print_syn_off() {
	f_mem_print("\n");
	f_mem_print("// synthesis translate_off\n");
	f_mem_print("\n");
}

void f_mem_close() {
	f_mem_print_end();
	fclose(mop);

	mem_count++;
}

void f_mem_print(const char *fmt, ...) {
	va_list ap;

	va_start(ap, fmt);

	vfprintf(mop, fmt, ap);
	fflush(mop);

	va_end(ap);
}

void f_temp_print(const char *fmt, ...) {
	va_list ap;

	va_start(ap, fmt);
	
	vfprintf(temp_fp, fmt, ap);
	fflush(temp_fp);

	va_end(ap);
}

void f_eth_print(const char *fmt, ...) {
	va_list ap;

	va_start(ap, fmt);

	vfprintf(eth_fp, fmt, ap);
	fflush(eth_fp);

	va_end(ap);
}

void f_eth_print_data(fixed_point data) {
	#if DATA_WIDTH == 32
		fprintf(eth_fp, "%02X\n", (data & 0xFF000000) >> 24);
		fprintf(eth_fp, "%02X\n", (data & 0x00FF0000) >> 16);
		fprintf(eth_fp, "%02X\n", (data & 0x0000FF00) >> 8);
		fprintf(eth_fp, "%02X\n", (data & 0x000000FF));
	#else
		fprintf(eth_fp, "%02X\n", (data & 0xFF00) >> 8);
		fprintf(eth_fp, "%02X\n", (data & 0x00FF));
	#endif
}

fixed_point f_adjusted_translate_number(float input) {
	fixed_point output;
	int temp1;
	int temp2;

	temp1 = input * pow(2, 30);

	#if DATA_WIDTH == 16
		//output = (temp1 & 0xFFFF0000) >> 16;
    output = temp1 >> 16;
    #if NEW_TRANSLATE == 1
			temp2 = temp1 & 0x0000FF00;
		
			if (temp2) {
				if (output >= 0) output += 1;
				else output -= 1;
			}
		#endif
	#else
		output = temp1;
	#endif

//	printf("haha: %.10f, %d, %08X, %d, %08X, %d, %0*X\n", input, temp1, temp1, temp2, temp2, output, BITWIDTH/4, output & MASK);
//	getchar();

	return output;	
}
