// -------------------------------------------------------
//
// Author: Thomas Lenart    
// Email:  thomas.lenart@es.lth.se
// Date:   March 2003
//
// -------------------------------------------------------
#include "main.h"

//#define IS_HARDWARE_ACCELERATED

#include <stdio.h>
#include <math.h>

#include "constants.h"
#include "subbandsyn.h"
#include "buffer.h"
#include "memhandler.h"
#include "print_mem.h"

#include "string.h"

#define MAX_OUTPUT_LIMIT	32767

fixed_point V[2][1024]; 

static /*const*/ fixed_point D[512] =	{
#if DATA_WIDTH == 32
#include "table_d"
#else
#include "table_d_16"
//#include "table_polyphase_window_16"
#endif
};

#if NEW_CODE == 1
	#if NEW_MDCT == 0
		static const fixed_point table_cos[64][32] =	{
			#if DATA_WIDTH == 32
				#include "table_mdct_cos_32"
			#else
//				#include "table_n_16"
				#include "table_mdct_cos_16"
			#endif
		};
	#endif
	#if NEW_MDCT == 1
		static const fixed_point table_cos[32][32] =	{
			#if DATA_WIDTH == 32
				#include "table_mdct_cos_32_new"
			#else
//				#include "table_n_16"
				#include "table_mdct_cos_16_new"
			#endif
		};
	#endif
	#if NEW_MDCT == 2 
// 		static const fixed_point c[10][33][2] =	{
// 			#if DATA_WIDTH == 32
// 				#include "table_fast_mdct_cos_32"
// 			#else
// //				#include "table_n_16"
// 				#include "table_fast_mdct_cos_16"
// 			#endif
// 		};
	#endif
#endif

void subband_synthesis_init(void) 
{
	int i;

	for (i = 0; i < 1024; i++) {
		V[0][i] = 0;
		V[1][i] = 0;
	}
}

int subband_synthesis(fixed_point *input, int ch, short *samples) 
{
	fixed_point				t[50];						// max temp variables used by subsyn_pre_gen.c
	int						i, j, ss, k;
	fixed_point				*vch = V[ch];
	static int				VOffset[2] = {64,64};
	register fixed_point	*VOffsetPtr, sum;
	fixed_point X_Prime[32];
	fixed_point				*bandPtr_fix;
	unsigned int			sample_index;
	fixed_point*			pN;
	fixed_point*			pD;
	register int			v_offset, index;
	signed int				sample_value;
	register int			lo, hi;
	#if NEW_POLYPHASE == 1
  int sample_32[1152];
  short sample_16[1152];
  #endif

	sample_index = ch;
	
	for (ss = 0; ss < MAX_SS; ss++) { 
		bandPtr_fix = &input[ss*MAX_SB];
		
		v_offset = (VOffset[ch] - 64) & 0x3ff;
		VOffset[ch] = v_offset;
		//bufOffsetPtr pointer to begining of the new V[]
		VOffsetPtr = &(V[ch][v_offset]);
		#ifdef DEBUG_SUBSYN
		FILE* file;
		if (ch_count == TARGET_CHANNEL || STEREO == 1) {
			#if DATA_WIDTH == 32
				file = fopen("debug_data/debug_mdct_input_32.txt", OUTPUT_FILE_TYPE);
			#else
				file = fopen("debug_data/debug_mdct_input_16.txt", OUTPUT_FILE_TYPE);
			#endif
			fprintf(file, "// This is frame %d, gr %d, ch %d, SS %d\n", frame_count, gr_count, ch_count, ss);

			for(i=0; i < 32; i++) { 
				fprintf(file, "%d: %d (%0*X)\n",i, bandPtr_fix[i],DATA_WIDTH/4, bandPtr_fix[i] & MASK);
			}
			fclose(file);
		}
		#endif
		#if NEW_CODE == 0
			#if DATA_WIDTH == 32
				#include "subsyn_pre_gen.c"
			#else
				#include "subsyn_pre_gen_16.c"
			#endif
		
			VOffsetPtr[16] = 0;
			for (i = 0; i < 16; i++) {
				VOffsetPtr[i] = -VOffsetPtr[32 - i];
				VOffsetPtr[i + 48] = VOffsetPtr[48 - i];
			}
		#else

			#if DATA_WIDTH == 32
				#include "subsyn_pre_gen_new_32.c"
			#else
				#include "subsyn_pre_gen_new_16.c"
			#endif

			// What we have in hardware
			#if (NEW_MDCT == 0)
				for (i = 16; i < 32; i++) {
					VOffsetPtr[i] = -VOffsetPtr[31 - i];
					VOffsetPtr[i + 16] = VOffsetPtr[63 - i + 16];
				}
			#else
				// what we think we should do
				VOffsetPtr[16] = 0;
				for (i = 0; i < 16; i++) {
					VOffsetPtr[i] = X_Prime[i+16];
// 					VOffsetPtr[i+17] = -X_Prime[31-i];
// 					VOffsetPtr[i+33] = -X_Prime[15-i];
// 					VOffsetPtr[i+49] = -X_Prime[i];
					VOffsetPtr[i+16] = -X_Prime[31-i];
					VOffsetPtr[i+32] = -X_Prime[15-i];
					VOffsetPtr[i+48] = -X_Prime[i];
				}
			#endif
		#endif
		#ifdef DEBUG_SUBSYN
			if (ch_count == TARGET_CHANNEL || STEREO == 1) {
				#if DATA_WIDTH == 32
					file = fopen("debug_data/debug_mdct_output_32.txt", OUTPUT_FILE_TYPE);
				#else
					file = fopen("debug_data/debug_mdct_output_16.txt", OUTPUT_FILE_TYPE);
				#endif
				fprintf(file, "// This is frame %d, gr %d, ch %d, SS %d\n", frame_count, gr_count, ch_count, ss);
				for(i=0; i < 64; i++){ 
					fprintf(file, "%d: %d (%0*X)\n",i, VOffsetPtr[i], DATA_WIDTH/4, VOffsetPtr[i] & MASK);
				}
				fclose(file);
			}
		#endif
			
		/*  S(i,j) = D(j+32i) * U(j+32i+((i+1)>>1)*64)  */
		/*  samples(i,j) = MWindow(j+32i) * bufPtr(j+32i+((i+1)>>1)*64)  */

		for (j=0; j<32; j++) {
			sum = 0;
			index = j;
			pD = &D[j];
#include "subsyn_post_gen.c"

			#if DATA_WIDTH == 32
				#if NEW_POLYPHASE == 1
					sample_value = (int)sum;
				#else
					sample_value = (int)((sum + 16384) >> 15);
				#endif
// 				sample_value = (int)((sum + 32768) >> 16);
			#else
				sample_value = (int)sum;
			#endif

			#if NEW_POLYPHASE == 1
				#if DATA_WIDTH == 32
  	      sample_32[sample_index] = sample_value*2;
    	  #else
      	  sample_16[sample_index] = (short)(sample_value << 1);
	      #endif
	     #endif
	     
			if (sample_value > MAX_OUTPUT_LIMIT) 
				sample_value = MAX_OUTPUT_LIMIT;

			if (sample_value < -MAX_OUTPUT_LIMIT) 
				sample_value = -MAX_OUTPUT_LIMIT;

			samples[sample_index] = (short)(sample_value);
// 			printf("haha: %d:, %08X, %04X\n", sample_index/2, sample_value, samples[sample_index] & 0x0000FFFF);
			sample_index += 2;
		}
	}
		
#ifdef DEBUG_SUBSYN
	{
		FILE* file;
		int i;
		if (frame_count >= START_FRAME) {
			if (ch_count == TARGET_CHANNEL || STEREO == 1) {
		
//		printf("Frame %d, gr, %d, ch %d SS %d\n", frame_count, gr_count, ch, ss);
/*
				#if DATA_WIDTH == 32
					file = fopen("debug_data/debug_from_subsyn_32.txt", OUTPUT_FILE_TYPE);
				#else
					file = fopen("debug_data/debug_from_subsyn_16.txt", OUTPUT_FILE_TYPE);
				#endif
*/
        #if DATA_WIDTH == 32
          file = fopen("debug_data/debug_from_subsyn_32.txt", "a");
        #else
          file = fopen("debug_data/debug_from_subsyn_16.txt", "a");
        #endif
				fprintf(file, "// This is frame %d, gr %d, ch %d\n", frame_count, gr_count, ch_count);
				for (i = 0; i < MAX_SB*MAX_SS; i++) {
					if (ch_count == 0) {
						#if NEW_POLYPHASE == 1
	            #if DATA_WIDTH == 32
								fprintf(file, "%d: %d (%0*X)\n", i, sample_32[2*i], DATA_WIDTH/4, sample_32[2*i] & MASK);	// only mono samples
    	        #else
        		    fprintf(file, "%d: %d (%0*X)\n", i, sample_16[2*i], DATA_WIDTH/4, sample_16[2*i] & MASK); // only mono samples
            	#endif
						#else
							fprintf(file, "%d: %d (%0*X)\n", i, samples[2*i], DATA_WIDTH/4, samples[2*i] & MASK);	// only mono samples
						#endif
					} else {
						#if NEW_POLYPHASE == 1
	            #if DATA_WIDTH == 32
								fprintf(file, "%d: %d (%0*X)\n", i, sample_32[2*i+1], DATA_WIDTH/4, sample_32[2*i+1] & MASK);
      	      #else
        		    fprintf(file, "%d: %d (%0*X)\n", i, sample_16[2*i+1], DATA_WIDTH/4, sample_16[2*i+1] & MASK);
            	#endif
						#else
							fprintf(file, "%d: %d (%0*X)\n", i, samples[2*i+1], DATA_WIDTH/4, samples[2*i+1] & MASK);
						#endif
					}
				}
				fclose(file);

				// for RMS check
        #if DATA_WIDTH == 32
          file = fopen("c_32.txt", "a");
        #else
          file = fopen("d_16.txt", "a");
        #endif
				for (i = 0; i < MAX_SB*MAX_SS; i++) {
					if (ch_count == 0) {
						#if NEW_POLYPHASE == 1
				       #if DATA_WIDTH == 32
				  	      //fprintf(file, "%0*X\n", 6, (samples[2*i] >> 8) & 0x00FFFFFF); // only mono samples
					        fprintf(file, "%d\n", (sample_32[2*i] >> 8)); // only mono samples
				       #else
        					fprintf(file, "%d\n", sample_16[2*i]); // only mono samples
				       #endif
						#else
							fprintf(file, "%0*X\n", DATA_WIDTH/4, samples[2*i] & MASK);	// only mono samples
						#endif
					} else {
						#if NEW_POLYPHASE == 1
				      #if DATA_WIDTH == 32
				  	     //fprintf(file, "%0*X\n", 6, (samples[2*i+1] >> 8) & 0x00FFFFFF); // only mono samples
					       fprintf(file, "%d\n", (sample_32[2*i+1] >> 8)); // only mono samples
				      #else
        				fprintf(file, "%d\n", sample_16[2*i+1]); // only mono samples
				      #endif
						#else
							fprintf(file, "%0*X\n", DATA_WIDTH/4, samples[2*i+1] & MASK);
						#endif
					}
				}
				fclose(file);
			}
		}
	}
#endif

	return 0;
}
